local HeadCount = LibStub("AceAddon-3.0"):NewAddon("HeadCount2", "AceEvent-3.0", "AceConsole-3.0")
if not HeadCount then return end
-- Load Locales
local L = LibStub("AceLocale-3.0"):GetLocale("HeadCount2")

local debug = true

function HeadCount:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HeadCount2DB", self:GetDBDefaults(), "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnDatabaseShutdown","Serialize")
	-- Setup constants
	self:InitializeConstants()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
end

function HeadCount:Serialize()
	-- Remove metadata methods from our data?
	-- Sicne AceDB-3.0 doesnt support OO like AceDB-2.0 did we need to do some magic here.
	local tracker = self:GetModule("RaidTracker")
	tracker:saveRaidListWrapper()
end

function HeadCount:OnEnable()
	-- Enhance the database
	self:AddClassMeta(self.db.profile)
	local tracker = self:GetModule("RaidTracker")
	if not self.db.profile.raidListWrapper then
		self.db.profile.raidListWrapper = self:NewInstance("RaidListWrapper")
	end
	tracker:SetRaidListWrapper(self.db.profile.raidListWrapper)
	tracker:recover()		-- process recovery for the raid tracker
	tracker:pruneRaids() 	-- prune old raids if applicable
	self:LogDebug(string.format(L["debug.mod.enable"], HeadCount.TITLE, HeadCount.VERSION))
	self:SetupOptions()
	self:SetupBroker()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function HeadCount:SetupBroker()
	local DataBroker = LibStub:GetLibrary("LibDataBroker-1.1")
	local launcher = nil
	if DataBroker then
		launcher = DataBroker:NewDataObject("HeadCount2", {
		    type = "launcher",
			OnTooltipShow = function(tooltip)
				if not tooltip or not tooltip.AddLine then return end
				tooltip:AddLine("|cFF9999FF" .. HeadCount.TITLE)
				tooltip:AddLine("")
				tooltip:AddLine(L["minimap.frame"], 0.2, 1, 0.2)
				tooltip:AddLine(L["minimap.configuration"], 0.2, 1, 0.2)
			end,
		    icon = "Interface\\Icons\\INV_Misc_Bone_HumanSkull_01",
		    OnClick = function(clickedframe, button)
				if button == "LeftButton" then
					-- Show Tracker Window
					LibStub("AceAddon-3.0"):GetAddon("HeadCount2"):GetModule("GUI"):ShowTracker()
				end
				if button == "RightButton" then
					-- Open Config screen
					LibStub("AceConfigDialog-3.0"):Open("HeadCount2")
				end
			end,
		})
	end
	local LibDBIcon = LibStub("LibDBIcon-1.0")
	if LibDBIcon and launcher then
		LibDBIcon:Register("HeadCount2", launcher, self.db.profile.minimap)
	end
end

function HeadCount:OnDisable()
	self:GetModule("RaidTracker"):RAID_ROSTER_UPDATE(true)
end

function HeadCount:Reset()
	-- Re-enhance the DB
	self:AddClassMeta(self.db.profile)
	local tracker = self:GetModule("RaidTracker")
	tracker:ResetWrapper()
end

function HeadCount:PLAYER_ENTERING_WORLD()
	local zone = GetRealZoneText()
	self:GetModule("RaidTracker"):recover()
	self:GetModule("RaidTracker"):RAID_ROSTER_UPDATE(true)
end

function HeadCount:ZONE_CHANGED_NEW_AREA()
	self:GetModule("RaidTracker"):processZoneChange()
end

function HeadCount:LogDebug(message, red, green, blue)
	if (self.db.profile.isDebugEnabled) then
		DEFAULT_CHAT_FRAME:AddMessage(message, red, green, blue)
	end
end

function HeadCount:LogInformation(message, red, green, blue)
	DEFAULT_CHAT_FRAME:AddMessage(message, red, green, blue)
end

function HeadCount:LogInformation(message)
	DEFAULT_CHAT_FRAME:AddMessage(message, 1.0, 1.0, 1.0)
end

function HeadCount:LogWarning(message)
	DEFAULT_CHAT_FRAME:AddMessage(message, 1.0, 1.0, 1.0)
end

function HeadCount:LogError(message, red, green, blue)
	UIErrorsFrame:AddMessage(message, red, green, blue)
end


--[[
	Channel Handling
--]]

function HeadCount:SendMessage(message, channel)
	assert(message, "Unable to send message because the message is nil.")
	assert(channel, "Unable to send message because the channel is nil.")
	if (channel == "GUILD") then
		HeadCount:SendGuildMessage(message)
	elseif (channel == "OFFICER") then
		HeadCount:SendOfficerMessage(message)
	elseif (channel == "PARTY") then
		HeadCount:SendPartyMessage(message)
	elseif (channel == "RAID") then
		HeadCount:SendRaidMessage(message)
	else
		-- TODO: eventually this will have to be modified for SAY/YELL vs arbitrary channels
		SendChatMessage(message, channel, nil, nil)
	end
end
-- Send a chat message to guild chat
-- @param message The message.
function HeadCount:SendGuildMessage(message)
	assert(message, "Unable to send guild message because the message is nil.")
	if (IsInGuild()) then
		SendChatMessage(message, "GUILD", nil, nil)
	end
end

-- Send a chat message to guild officer chat.
-- @param message The message.
function HeadCount:SendOfficerMessage(message)
	assert(message, "Unable to send guild officer message because the message is nil.")
	if (IsInGuild()) then
		SendChatMessage(message, "OFFICER", nil, nil)
	end
end

-- Send a chat message to party chat.
-- @param message The message.
function HeadCount:SendPartyMessage(message)
	assert(message, "Unable to send party message because the message is nil.")
	local isPresentInRaid = UnitInRaid("player")
	local isPresentInParty = (GetNumPartyMembers() > 0)
	if ((isPresentInRaid) or (isPresentInParty)) then
		SendChatMessage(message, "PARTY", nil, nil)
	end
end

-- Send a chat message to raid chat.
-- @param message The message.
function HeadCount:SendRaidMessage(message)
	assert(message, "Unable to send raid message because the message is nil.")
	if (1 == UnitInRaid("player")) then
		SendChatMessage(message, "RAID", nil, nil)
	end
end

-- Send a whisper to the recipient.
-- @param message The message.
-- @param recipient The recipient.
function HeadCount:SendWhisper(message, recipient)
	assert(message, "Unable to send whisper message because the message is nil.")
	assert(recipient, "Unable to send whisper message because the recipient is nil.")
	SendChatMessage(message, "WHISPER", nil, recipient)
end